import asyncio
import zipfile
from js import document, FileReader, Uint8Array
from pyodide.ffi import create_proxy
import io
import pandas as pd
import json
from pyscript import display

t = document.getElementById("dataContainer")

def show_message(message):
    t.innerHTML += f'<p class="small-font">{message}</p>'

def show_raw_message(message):
    t.innerHTML += message

async def process_file(event):
    t.innerHTML = '<p class="small-font">Loading...</p>'

    try:
        fileList = event.target.files.to_py()
        all_data = []
        
        for f in fileList:
            data = Uint8Array.new(await f.arrayBuffer())
            mf = io.BytesIO(bytearray(data))
            
            with zipfile.ZipFile(mf, "r") as zf:
                nl = zf.namelist()
                
                for file_name in nl:
                    if file_name.endswith('.json') and 'Video' not in file_name and 'Streaming_History_Audio' in file_name:
                        with zf.open(file_name) as file:
                            data = json.load(io.TextIOWrapper(file, 'utf-8'))
                            all_data.extend(data)

        if not all_data:
            show_message('No valid data found in the uploaded file.')
            t.innerHTML = t.innerHTML[36::]
            return
        
        spotify = pd.DataFrame(all_data)
        date = spotify.sort_values(by='ts')
        show_message(f'The file contains data from {str(date["ts"].iloc[0])[0:10]} all the way to {str(date["ts"].iloc[-1])[0:10]}')

        songs = spotify.groupby(['spotify_track_uri']).ngroups
        show_message(f'You listened to a total of {songs} different songs')
        
        totaltime = spotify['ms_played'].sum() / 1000 / 60
        show_message(f'You listened to a total of {int(totaltime)} minutes ({int(totaltime/60)} hours or {int(totaltime/60/24)} days) of music')

        
        # Calculate and display top artists
        top10_artists = spotify.groupby('master_metadata_album_artist_name')['ms_played'].sum().sort_values(ascending=False).head(10)
        top10artiststext = ''.join([f'<p class="small-font">{i+1}. {artist} ({int(minutes/60000)} minutes)</p>' for i, (artist, minutes) in enumerate(top10_artists.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened artists - By time listened</button><div class="panel">{top10artiststext}</div>')
        
        # Calculate and display top songs
        top10_songs = spotify.groupby('master_metadata_track_name')['ms_played'].sum().sort_values(ascending=False).head(10)
        top10songstext = ''.join([f'<p class="small-font">{i+1}. {song} ({int(minutes/60000)} minutes)</p>' for i, (song, minutes) in enumerate(top10_songs.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened songs - By time listened</button><div class="panel">{top10songstext}</div>')
        
        # Calculate and display top days
        spotify['ts'] = pd.to_datetime(spotify['ts'])
        daily_playtime = spotify.groupby(spotify['ts'].dt.date)['ms_played'].sum() / (1000 * 60)
        top10_days = daily_playtime.sort_values(ascending=False).head(10)
        top10daystext = ''.join([f'<p class="small-font">{i+1}. {day} ({int(minutes)} minutes)</p>' for i, (day, minutes) in enumerate(top10_days.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened days - By time listened</button><div class="panel">{top10daystext}</div>')
        
        # Calculate and display top albums
        topalbum = spotify.groupby(['master_metadata_album_album_name', 'master_metadata_album_artist_name'])['ms_played'].sum().sort_values(ascending=False).head(10)
        topalbumtext = ''.join([f'<p class="small-font">{i+1}. {album} by {artist} ({int(minutes/60000)} minutes)</p>' for i, ((album, artist), minutes) in enumerate(zip(topalbum.index, topalbum))])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened albums - By time listened</button><div class="panel">{topalbumtext}</div>')
        
        # Calculate and display top weekdays
        total_minutos_por_dia = spotify.groupby(spotify['ts'].dt.day_name())['ms_played'].sum()
        weekdays_playtime = total_minutos_por_dia.sort_values(ascending=False)
        weekdays_text = ''.join([f'<p class="small-font">{i+1}. {day}s ({int(minutes/60000)} minutes)</p>' for i, (day, minutes) in enumerate(weekdays_playtime.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened weekdays - By time listened</button><div class="panel">{weekdays_text}</div>')
        
        # Calculate and display top hours
        total_minutos_por_hora = spotify.groupby(spotify['ts'].dt.hour)['ms_played'].sum()
        top10_hours = total_minutos_por_hora.sort_values(ascending=False).head(10)
        top10hourstext = ''.join([f'<p class="small-font">{i+1}. {hour}:00 ({int(minutes/60000)} minutes)</p>' for i, (hour, minutes) in enumerate(top10_hours.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened hours - By time listened</button><div class="panel">{top10hourstext}</div>')
        
        # Calculate and display top months
        monthly_listening_time = spotify.groupby(spotify['ts'].dt.month_name())['ms_played'].sum() / (1000 * 60)
        monthly_playtime = monthly_listening_time.sort_values(ascending=False)
        monthly_text = ''.join([f'<p class="small-font">{i+1}. {month} ({int(minutes)} minutes)</p>' for i, (month, minutes) in enumerate(monthly_playtime.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened months - By time listened</button><div class="panel">{monthly_text}</div>')
        
        # Calculate and display most skipped songs
        skipped_songs = spotify[spotify['reason_end'] == 'fwdbtn']
        song_skip_counts = skipped_songs.groupby('master_metadata_track_name').size()
        top10_skipped_songs = song_skip_counts.sort_values(ascending=False).head(10)
        skipped_songs_text = ''.join([f'<p class="small-font">{i+1}. {song} ({count} skips)</p>' for i, (song, count) in enumerate(top10_skipped_songs.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most skipped songs</button><div class="panel">{skipped_songs_text}</div>')

        # Calculate and display top years
        spotify['ts'] = pd.to_datetime(spotify['ts'])
        yearly_listening_time = spotify.groupby(spotify['ts'].dt.year)['ms_played'].sum() / (1000 * 60)
        top_years = yearly_listening_time.sort_values(ascending=False).head(10)
        
        # Format top years in the specified format
        top_years_text = ''.join([f'<p class="small-font">{i+1}. {year} ({int(minutes)} minutes)</p>' for i, (year, minutes) in enumerate(top_years.items())])
        show_raw_message(f'<button class="accordion" style="color: white;">Your most listened years - By time listened</button><div class="panel">{top_years_text}</div>')
        

        t.innerHTML = t.innerHTML[36::]

    except Exception as e:
        show_raw_message(f'There was an error processing the file: {str(e)}')
        t.innerHTML = t.innerHTML[36::]

def main():
    file_event = create_proxy(process_file)
    e = document.getElementById("myfile")
    e.addEventListener("change", file_event, False)

main()
